@@ -6,6 +6,7 @@ from cliff.command import Command
 from cliff.lister import Lister
 from cliff.show import ShowOne
 
+from .. import formatter
 from ..utils import tabulate, is_tty
 
 
@@ -42,6 +43,10 @@ class ListInstance(Lister):
             query_params['type'] = 'advanced'
 
         data = self.app.session.get( '/instances', params=query_params)
+
+        for instance in data:
+            instance['memory'] = formatter.format_ram(int(instance['memory']))
+
         return tabulate(data)
 
 
@@ -55,6 +60,11 @@ class CreateInstance(ShowOne):
             metavar='<name>',
             help='Instance name'
         )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help='Instance description'
+        )
         parser.add_argument(
             '--instance-type',
             required=True,
@@ -92,6 +102,19 @@ class CreateInstance(ShowOne):
             default=None,
             action='store_true'
         )
+        parser.add_argument(
+            '--with-floating-ip',
+            help='Create and associate a Floating IP automatically',
+            default=None,
+            action='store_true'
+        )
+        parser.add_argument(
+            '--additional-volumes',
+            metavar='<additional_volumes>',
+            default=[],
+            action='append',
+            help='attach additional volume automatically, e.g --additional-volumes 500'
+        )
 
         parser.add_argument(
             '--advanced',
@@ -145,9 +168,22 @@ class CreateInstance(ShowOne):
             payload['ssh_key_name'] = parsed_args.ssh_key
         if parsed_args.security_group:
             payload['security_groups'] = parsed_args.security_group
+        if parsed_args.additional_volumes:
+            payload['additional_volumes'] = [
+                {
+                    "size": volume_size,
+                    "volume_type": "SSD",
+                } for volume_size in parsed_args.additional_volumes
+            ]
+        if parsed_args.with_floating_ip:
+            payload['has_floating_ip'] = parsed_args.with_floating_ip
+        if parsed_args.description:
+            payload['description'] = parsed_args.description
 
         data = self.app.session.post('/instances', payload)
 
+        data['memory'] = formatter.format_ram(int(data['memory']))
+
         return tabulate(data)
 
 
@@ -167,6 +203,8 @@ class ShowInstance(ShowOne):
         uri = '/instances/%s' % parsed_args.instance
         data = self.app.session.get(uri)
 
+        data['memory'] = formatter.format_ram(int(data['memory']))
+
         return tabulate(data)
 
 
@@ -258,7 +296,7 @@ class StopInstance(Command):
 
     def take_action(self, parsed_args):
         uri = '/instances/%s/state-transitions' % parsed_args.instance
-        self.app.session.post(uri, payload=dict(new_state="STOPPED"))
+        self.app.session.post(uri, payload=dict(new_state="SHUTOFF"))
 
 
 class SuspendInstance(Command):
@@ -275,7 +313,7 @@ class SuspendInstance(Command):
 
     def take_action(self, parsed_args):
         uri = '/instances/%s/state-transitions' % parsed_args.instance
-        self.app.session.post(uri, payload=dict(new_state="SUSPEND"))
+        self.app.session.post(uri, payload=dict(new_state="SUSPENDED"))
 
 
 class ResumeInstance(Command):
@@ -367,7 +405,7 @@ class ShowInstanceVolume(ShowOne):
         return tabulate(data)
 
 
-class AttachInstanceVolume(Command):
+class AttachInstanceVolume(ShowOne):
     """Attach instance volume."""
 
     def get_parser(self, prog_name):
@@ -390,7 +428,9 @@ class AttachInstanceVolume(Command):
         uri = '/instances/%s/volumes' % parsed_args.instance
         payload = {'id': parsed_args.volume}
 
-        self.app.session.post(uri, payload)
+        data = self.app.session.post(uri, payload)
+
+        return tabulate(data)
 
 
 class DetachInstanceVolume(Command):
@@ -437,7 +477,7 @@ class ListInstanceInterface(Lister):
         return tabulate(data)
 
 
-class AttachInterface(Command):
+class AttachInterface(ShowOne):
     """Attach interface."""
 
     def get_parser(self, prog_name):
@@ -459,7 +499,8 @@ class AttachInterface(Command):
     def take_action(self, parsed_args):
         payload = {'net': parsed_args.network}
         uri = '/instances/%s/interfaces' % parsed_args.instance
-        self.app.session.post(uri, payload)
+        data = self.app.session.post(uri, payload)
+        return tabulate(data)
 
 
 class DetachInterface(Command):
@@ -473,17 +514,17 @@ class DetachInterface(Command):
             help='Instance name or ID'
         )
         parser.add_argument(
-            '--port-id',
+            '--interface-id',
             required=True,
-            metavar='<port>',
-            help='Port ID',
+            metavar='<interface>',
+            help='Interface ID',
 
         )
 
         return parser
 
     def take_action(self, parsed_args):
-        uri = '/instances/%s/interfaces/%s' %(parsed_args.instance, parsed_args.port_id)
+        uri = '/instances/%s/interfaces/%s' %(parsed_args.instance, parsed_args.interface_id)
         self.app.session.delete(uri)
 
 
@@ -494,22 +535,33 @@ class UpdateInstance(ShowOne):
         parser.add_argument(
             'instance',
             metavar='<instance>',
-            help=('Instance ID'),
+            help='Instance ID or name',
         )
         parser.add_argument(
             '--name',
             metavar='<name>',
-            required=True,
-            help='New instance name.',
+            required=False,
+            help=('Instance new name, should be unique'),
         )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            required=False,
+            help=('Instance description'),
+        )
+
         return parser
 
     def take_action(self, parsed_args):
         uri = '/instances/%s' % parsed_args.instance
-        payload = {
-            "name": parsed_args.name
-        }
-        data = self.app.session.put(uri, payload)
+        payload = {}
+        if parsed_args.name:
+            payload['name'] = parsed_args.name
+        if parsed_args.description:
+            payload['description'] = parsed_args.description
+        if not payload:
+            raise Exception("At least one of --name or --description should be provided")
+        data = self.app.session.patch(uri, payload)
 
         return tabulate(data)
 
